# aira/commit_manager.py
class CommitManager:
    def __init__(self, config):
        self.config = config

    # 今後のフェーズで実装するメソッド
    def create_commit_message(self, changes, llm_client):
        pass

    def stage_files(self):
        pass

    def commit(self):
        pass


# aira/config.py
import yaml

class Config:
    def __init__(self, file_path):
        with open(file_path, "r") as f:
            self.config = yaml.safe_load(f)

    def get(self, key, default=None):
        """
        設定値を取得する。
        key: ドット区切りのキー (例: "gaiah.repo.create_repo")
        default: キーが存在しない場合に返すデフォルト値 (省略可)
        """
        keys = key.split(".")
        value = self.config
        for k in keys:
            if isinstance(value, dict):
                value = value.get(k)
            if value is None:
                return default
        return value


# aira/aira.py
from loguru import logger
from .repository_manager import RepositoryManager
from .commit_manager import CommitManager
from gaiah.gaiah import Gaiah
from gaiah.cli import load_config
from harmon_ai.harmon_ai import HarmonAI
import pprint

from litellm import completion
import os

from dotenv import load_dotenv

load_dotenv(verbose=True)

class Aira:
    def __init__(self, config_path=".aira/config.yml"):
        logger.debug(f"aira config_path : {config_path}")
        self.aira_config = load_config(config_path)
        self.gaiah_config = {"gaiah" : self.aira_config["aira"]["gaiah"]}
        
        self.repo_manager = RepositoryManager(self.gaiah_config, self.aira_config)
        self.commit_manager = CommitManager(self.aira_config)

    def make_repo(self):
        logger.info("Gaiah run : {}".format(self.aira_config["aira"]["gaiah"]["run"]))
        if self.aira_config["aira"]["gaiah"]["run"]:
            logger.info("Gaiah section starting...")
            
            if not self.repo_manager.is_initialized():
                logger.info("Initializing repository...")
                self.repo_manager.init()
                self.gaiah_config["gaiah"] = self.merge_configs(self.gaiah_config["gaiah"], self.gaiah_config["gaiah"]["init"])
                
                gaiah = Gaiah(self.gaiah_config)
                gaiah.run()  # Gaiahの実行
            else:
                logger.info("Develop repository...")
                self.gaiah_config["gaiah"] = self.merge_configs(self.gaiah_config["gaiah"], self.gaiah_config["gaiah"]["dev"])
            


        logger.info("Harmon AI run : {}".format(self.aira_config["aira"]["harmon_ai"]["run"]))
        if self.aira_config["aira"]["harmon_ai"]["run"]:
            logger.info("HarmonAI section starting...")
                       
            output_dir = self.aira_config["aira"]["harmon_ai"]["development"]["output_dir"]
            # output_dir = self.aira_config.get("repository_summary_output_dir", ".aira")
            
            # リポジトリ概要を生成
            instructions_prompt_path = self.aira_config["aira"]["harmon_ai"]["instructions_prompt"]
            repository_summary = self.repo_manager.generate_repository_summary(
                instructions_prompt_path
            )
            
            # リポジトリ概要を保存
            output_file="repository_summary_raw.md"
            self.repo_manager.save_repository_summary(repository_summary, output_dir, output_file)
            
            repository_summary_raw = os.path.join(output_dir, output_file)
            with open(repository_summary_raw, "r", encoding="utf-8") as f:
                repository_summary =  f.read()

            # # README を生成
            readme_prompt_template_path = r".aira\prompt_readme.md"
            readme = self.generate_readme(
                repository_summary, readme_prompt_template_path
            )
            
            output_file = "sections_template_llm.md"
            self.repo_manager.save_repository_summary(readme, output_dir, output_file)
            
            harmon_ai_config = {"harmon_ai" : self.aira_config["aira"]["harmon_ai"]}
            harmon_ai_config["harmon_ai"]["product"]["sections_content_file"] = output_file
            harmon_ai = HarmonAI(config=harmon_ai_config)
            harmon_ai.run()  # HarmonAIの実行
    
    def run(self):
        logger.info("Gaiah run : {}".format(self.aira_config["aira"]["gaiah"]["run"]))
        if self.aira_config["aira"]["gaiah"]["run"]:
            logger.info("Gaiah section starting...")
            
            if self.repo_manager.is_initialized():
                logger.info("Develop repository...")
                self.gaiah_config["gaiah"] = self.merge_configs(self.gaiah_config["gaiah"], self.gaiah_config["gaiah"]["dev"])
                gaiah = Gaiah(self.gaiah_config)
                gaiah.run()  # Gaiahの実行

    def co_and_merge_branches(self):
        if self.aira_config["aira"]["gaiah"]["run"]:
            logger.info("Gaiah section starting...")
            
            if self.repo_manager.is_initialized():
                logger.info("Develop repository...")
                self.gaiah_config["gaiah"] = self.merge_configs(self.gaiah_config["gaiah"], self.gaiah_config["gaiah"]["dev"])
                gaiah = Gaiah(self.gaiah_config)
                gaiah.checkout_and_merge_branches()  # Gaiahの実行
                gaiah.push()  # Gaiahの実行
    
    def merge_configs(self, base_config, override_config):
        merged_config = base_config.copy()
        for key, value in override_config.items():
            if isinstance(value, dict):
                merged_config[key] = self.merge_configs(merged_config.get(key, {}), value)
            else:
                merged_config[key] = value
        return merged_config
    
    
    def generate_readme(self, repository_summary, prompt_template_path=None):
        """
        リポジトリの概要とテンプレートからREADMEを生成する
        """
        try:
            sections_content_path = r".harmon_ai\sections_template.md"
            with open(sections_content_path, "r", encoding="utf-8") as f:
                template = f.read()

            # プロンプトテンプレートファイルの読み込み
            if prompt_template_path:
                try:
                    with open(prompt_template_path, "r", encoding="utf-8") as f:
                        prompt_template = f.read()
                except FileNotFoundError:
                    logger.warning(f"Prompt template file not found: {prompt_template_path}. Using default template.")
                    prompt_template = "下記のテンプレートに従ってリポジトリの概要からREADMEを作成して\n\n# テンプレート\n\n[テンプレート]\n\n# 概要\n\n[概要]"
            else:
                prompt_template = "下記のテンプレートに従ってリポジトリの概要からREADMEを作成して\n\n# テンプレート\n\n[テンプレート]\n\n# 概要\n\n[概要]"

            prompt = prompt_template.replace("[readme_template]", template).replace("[readme_abst]", repository_summary)

            response = completion(
                model=self.aira_config["aira"]["llm"]["model"],
                messages=[{"role": "user", "content": prompt}]
            )
            return response["choices"][0]["message"]["content"]
        except Exception as e:
            logger.error(f"Error generating README: {e}")
            return None


# aira/repository_manager.py
import subprocess
from loguru import logger
import os
from gaiah.utils import run_command, tqdm_sleep
from litellm import completion

# import litellm
# litellm.set_verbose=True

class RepositoryManager:
    def __init__(self, config, aira_config):
        self.config = config
        self.aira_config = aira_config

    def is_initialized(self):
        repo_dir = self.config["gaiah"]["local"]["repo_dir"]
        logger.info(f"repo_dir is {repo_dir}")

        # ディレクトリが存在するか確認
        if not os.path.exists(repo_dir):
            return False

        return ".git" in os.listdir(repo_dir)

    def init(self):
        repo_dir = self.config["gaiah"]["local"]["repo_dir"]
        os.makedirs(repo_dir, exist_ok=True)
        run_command(command=["git", "init"], cwd=repo_dir)
        logger.info(f"Initialized repository at {repo_dir}")

    def generate_repository_summary(self, instructions_prompt_path):
        """
        リポジトリの概要をLLMで生成する
        """
        try:
            with open(instructions_prompt_path, "r", encoding="utf8") as f:
                instructions_prompt = f.read()

            response = completion(
                model=self.aira_config["aira"]["llm"]["model"],
                messages=[{"role": "user", "content": instructions_prompt}]
            )
            return response["choices"][0]["message"]["content"]
        except Exception as e:
            logger.error(f"Error generating repository summary: {e}")
            return None
        
    def save_repository_summary(self, summary, output_dir=".aira", output_file="repository_summary_raw.md"):
        """
        リポジトリ概要をファイルに保存する
        """
        os.makedirs(output_dir, exist_ok=True)  # 出力ディレクトリが存在しない場合は作成
        file_path = os.path.join(output_dir, output_file)

        try:
            with open(file_path, "w", encoding="utf-8") as f:
                f.write(summary)
            logger.info(f"Saved repository summary to {file_path}")
        except Exception as e:
            logger.error(f"Error saving repository summary: {e}")


import argparse
from art import tprint
from loguru import logger
from gaiah.gaiah import Gaiah
from gaiah.cli import load_config

from harmon_ai.harmon_ai import HarmonAI

import sys
import os
import shutil
import yaml
from .aira import Aira
logger.configure(
    handlers=[
        {
            "sink": sys.stderr,
            "format": "<green>{time:YYYY-MM-DD HH:mm:ss.SSS}</green> | <level>{level:<8}</level> | <cyan>{name:<35}:{function:<35}:{line:<5}</cyan> | <level>{message}</level>",
            "colorize": True,
        }
    ]
)

def parse_arguments():
    """
    コマンドライン引数を解析する
    """
    parser = argparse.ArgumentParser(description='AIRA - AI-Integrated Repository for Accelerated Development')
    parser.add_argument('--config', default='.aira/config.dev.yml', help='設定ファイルのパス')
    parser.add_argument('--mode', default='commit', help='処理モード')
    return parser.parse_args()

def main():
    tprint("!  Welcome  to  AIRA  !")

    args = parse_arguments()
    aira = Aira(args.config)
    if(args.mode == "make"):
        logger.info("mode is << make >>")
        aira.make_repo()
        aira.run()
        aira.co_and_merge_branches()
        
    if(args.mode == "commit"):
        logger.info("mode is << commit >>")
        aira.run()
        pass
        

if __name__ == "__main__":
    main()


from setuptools import setup, find_packages
import re

def get_version():
    with open("aira/__init__.py", "r") as f:
        content = f.read()
        version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]", content, re.MULTILINE)
        if version_match:
            return version_match.group(1)
        else:
            raise RuntimeError("Unable to find version string.")

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name='aira',
    version=get_version(),
    packages=find_packages(),
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Version Control",
    ],
    package_data={
        'aira': ['README.md',
                  'requirements.txt',
                  'template/*'], 
    },
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/your_username/aira",
    install_requires=[
            'python-dotenv',
            'termcolor',
            'art',
            'loguru',
            'tqdm',
            'harmon_ai',
            'gaiah_toolkit',
        ],
    entry_points={
        'console_scripts': [
            'aira=aira.cli:main',
        ],
    },
)